#include <cmath>
#include <fstream>
#include <iostream>

/**
 * A simple enum to indicate if we are on a "red" or "blue" row
 */
enum RowType {
    RedRow,
    BlueRow
};

/**
 * A simple enum to track the filter type
 */
enum FilterType {
    RedFilter   = 0,
    GreenFilter = 1,
    BlueFilter  = 2
};

/**
 * This is a simple program that generates a 320 x 240, Bayer pattern
 * pixel database file for use by the DIRSIG4 data-driven detector
 * array model.  A pixel database file includes the pixel line-of-sight
 * (LOS) vectors (as an X/Y angle pair) and optional spectral response
 * information.  The program creates the pixel databse using a minimal
 * set of inputs including the focal length, pixel pitch and number of
 * pixels in each dimension.  The Bayer pattern is imposed by the
 * assignment of spectral response functions to each pixel via a channel
 * index.
 * 
 * This program can be easily modified to model a different size array,
 * to use a different focal length, or to use a different color filter
 * array (CFA) pattern (other than Bayer).
 */
int
main( int argCount, char ** argVector )
{
    // the focal length is 100 mm
    const double focalLength = 100.0e-03;

    // the array size is 320 x 240
    const int xArraySize = 320;
    const int yArraySize = 240;

    // the pixel pitch is 40 microns
    const double pixelPitch = 40.0e-06;

    // the pixel IFOV can be computed
    const double pixelIfov = atan( pixelPitch / focalLength );

    // open the output pixel database file
    std::ofstream pixelFile( "pixels.dat" );

    // iterate through all the Y rows in this array
    for( int yy = 0; yy < yArraySize; yy++ ) {

        // figure out if this "red" or "blue" row
        RowType rowType;
        if(( yy % 2 ) == 0 ) {
            rowType = RedRow;
        }
        else {
            rowType = BlueRow;
        }

        // iterate through all the X pixels in this row
        for( int xx = 0; xx < xArraySize; xx++ ) {

            // compute the pixel X/Y location
            double xLoc = ( xx - ( xArraySize / 2.0 )) * pixelPitch;
            double yLoc = ( yy - ( yArraySize / 2.0 )) * pixelPitch;

            // compute the pixel X/Y angles
            double xAngle = atan( xLoc / focalLength );
            double yAngle = atan( yLoc / focalLength );

            // figure out what filter it gets
            FilterType filterType;
            if( rowType == RedRow ) {
                if(( xx % 2 ) == 0 ) {
                    filterType = RedFilter;
                }
                else {
                    filterType = GreenFilter;
                }
            }
            else if( rowType == BlueRow ) {
                if(( xx % 2 ) == 1 ) {
                    filterType = BlueFilter;
                }
                else {
                    filterType = GreenFilter;
                }
            }

            // write this pixel out to the file
            pixelFile
                << xx << " " << yy << " "
                << xAngle << " " << yAngle << " "
                << pixelIfov << " " << pixelIfov << " "
                << static_cast< int >( filterType ) << std::endl;
        }
    }
}
